from .base import ScalarVariable
from .global_global_step import global_step
from .global_global_step import get_value as current_step
from .global_keep_prob import keep_prob
from .global_batch_size import batch_size
from .utils import create_global_scalars
